/*
 * Name:    mp_motor.c
 *
 * Purpose: Python module interface to generic MX motor functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_motor.h"
#include "mx_handle.h"
#include "mp.h"

MX_EXPORT PyObject *
MpPrivate_motor_is_busy_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status, busy;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_is_busy( record, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", busy );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_move_absolute_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double destination;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &destination );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_move_absolute( record, destination, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_move_relative_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double relative_position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld",
						&handle, &relative_position );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_move_relative( record, relative_position, 0 );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_move_absolute_with_flags_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status, flags;
	double destination;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ldi",
					&handle, &destination, &flags );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_move_absolute( record, destination, flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_move_relative_with_flags_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status, flags;
	double relative_position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ldi",
					&handle, &relative_position, &flags );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_move_relative( record, relative_position, flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_wait_for_motor_stop_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	int flags;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "li", &handle, &flags );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_wait_for_motor_stop( record, flags );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_position_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_position( record, &position );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", position );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_position_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double set_position;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &set_position );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_position( record, set_position );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_soft_abort_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_soft_abort( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_immediate_abort_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_immediate_abort( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_positive_limit_hit_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status, limit_hit;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_positive_limit_hit( record, &limit_hit );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", limit_hit );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_negative_limit_hit_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status, limit_hit;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_negative_limit_hit( record, &limit_hit );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", limit_hit );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_find_home_position_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long direction;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &direction );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_find_home_position( record, (int) direction );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_constant_velocity_move_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	long direction;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ll", &handle, &direction );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_constant_velocity_move( record, (int) direction );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_speed_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_speed( record, &speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", speed );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &speed );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_speed( record, speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_base_speed_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_base_speed( record, &speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", speed );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_base_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &speed );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_base_speed( record, speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_maximum_speed_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_maximum_speed( record, &speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", speed );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_maximum_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double speed;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "ld", &handle, &speed );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_maximum_speed( record, speed );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_save_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_save_speed( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_restore_speed_cmd( PyObject *self, PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_restore_speed( record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_speed_between_positions_cmd( PyObject *self,
							PyObject *args )
{
	MX_RECORD *record;
	long handle;
	int parse_status;
	double position1, position2, time_for_move;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lddd", &handle,
				&position1, &position2, &time_for_move );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_set_speed_between_positions( record,
						position1, position2,
						time_for_move );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_accel_type_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	int acceleration_type;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_acceleration_type( record,
							&acceleration_type );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", acceleration_type );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_accel_time_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double acceleration_time;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_acceleration_time( record,
						&acceleration_time );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", acceleration_time );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_accel_distance_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status;
	double acceleration_distance;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_acceleration_distance( record,
						&acceleration_distance );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "d", acceleration_distance );

	return result;
}

MX_EXPORT PyObject *
MpPrivate_motor_get_accel_parameters_cmd( PyObject *self, PyObject *args )
{
	PyObject *parameters_tuple, *parameters_element;
	MX_RECORD *record;
	long handle;
	int i, parse_status;
	double parameter_array[MX_MOTOR_NUM_ACCELERATION_PARAMS];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_get_acceleration_parameters(
						record, parameter_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Create a new tuple to contain the acceleration parameters. */

	parameters_tuple = PyTuple_New( MX_MOTOR_NUM_ACCELERATION_PARAMS );

	if ( parameters_tuple == NULL )
		return NULL;

	/* Copy the acceleration parameters to the tuple. */

	for ( i = 0; i < MX_MOTOR_NUM_ACCELERATION_PARAMS; i++ ) {

		parameters_element = PyFloat_FromDouble( parameter_array[i] );

		if ( parameters_element == NULL ) {
			Py_DECREF( parameters_tuple );
			return NULL;
		}

		PyTuple_SET_ITEM( parameters_tuple, i, parameters_element );
	}

	return parameters_tuple;
}

MX_EXPORT PyObject *
MpPrivate_motor_set_accel_parameters_cmd( PyObject *self, PyObject *args )
{
	PyObject *parameters_tuple, *parameters_element;
	MX_RECORD *record;
	long handle;
	int i, parse_status, type_status, element_status, tuple_length;
	double parameter_array[MX_MOTOR_NUM_ACCELERATION_PARAMS];
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "lO",
					&handle, &parameters_tuple );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	/* Verify that the object we have been passed is a tuple. */

	type_status = PyTuple_Check( parameters_tuple );

	if ( type_status == 0 )
		return NULL;

	/* Find out how many object are in the tuple. */

	tuple_length = PyTuple_Size( parameters_tuple );

	if ( tuple_length < 0 )
		return NULL;

	/* Check to see if we have received the correct number of parameters
	 * in the parameter object.
	 */

	if ( tuple_length != MX_MOTOR_NUM_ACCELERATION_PARAMS ) {

		/* Raise an exception if we have the wrong number
		 * of parameters.
		 */

		MpPrivateErr_SetString( PyExc_IndexError,
			"The method '%s' was passed the wrong number of "
			"parameters, namely %d.  "
			"There should be %d parameters.",
			tuple_length, MX_MOTOR_NUM_ACCELERATION_PARAMS );

		return NULL;
	}

	/* Walk through the tuple and get the values of the individual
	 * parameters.
	 */

	for ( i = 0; i < MX_MOTOR_NUM_ACCELERATION_PARAMS; i++ ) {

		/* Get the next tuple element. */

		parameters_element = PyTuple_GetItem( parameters_tuple, i );

		if ( parameters_element == NULL )
			return NULL;

		/* Verify that the element is a PyFloatObject. */

		element_status = PyFloat_Check( parameters_element );

		if ( element_status == 0 )
			return NULL;

		/* Copy the tuple element value to the parameter array. */

		parameter_array[i] = PyFloat_AsDouble( parameters_element );
	}

	mx_status = mx_motor_set_acceleration_parameters(
						record, parameter_array );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_motor_home_search_succeeded_cmd( PyObject *self, PyObject *args )
{
	PyObject *result;
	MX_RECORD *record;
	long handle;
	int parse_status, home_search_succeeded;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "l", &handle );

	if ( parse_status == 0 )
		return NULL;

	mx_status = MpPrivate_get_record_from_handle( handle, &record );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	mx_status = mx_motor_home_search_succeeded( record,
						&home_search_succeeded );

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	result = Py_BuildValue( "i", home_search_succeeded );

	return result;
}

/*** The following motors operate on multiple motors at once. ***/

#ifdef FREE_ARRAYS
#undef FREE_ARRAYS
#endif

#define FREE_ARRAYS \
		do { \
			mx_free( motor_record_array ); \
			mx_free( position_array ); \
		} while (0)

MX_EXPORT PyObject *
MpPrivate_multiple_motors_move_absolute_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "multiple_motors_move_absolute";

	PyObject *motors_tuple, *positions_tuple;
	PyObject *motor_object, *position_object;
	int i, parse_status, num_motors, num_positions;
	MX_RECORD **motor_record_array, *motor_record;
	double *position_array;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "OO",
					&motors_tuple, &positions_tuple );

	if ( parse_status == 0 )
		return NULL;

	/* The first object passed should be a tuple of motor objects. */

	if ( PyTuple_Check( motors_tuple ) == 0 ) {
		MpPrivateErr_SetString( PyExc_TypeError,
		"The first argument to '%s' should be a tuple, but it is not.",
			cname );
		return NULL;
	}

	/* The second object passed should be a tuple of motor positions. */

	if ( PyTuple_Check( positions_tuple ) == 0 ) {
		MpPrivateErr_SetString( PyExc_TypeError,
		"The second argument to '%s' should be a tuple, but it is not.",
			cname );
		return NULL;
	}

	/* Figure out the number of motors by getting the number of objects
	 * in the motor tuple.
	 */

	num_motors = PyTuple_Size( motors_tuple );

	if ( num_motors <= 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
			"The motors tuple passed to '%s' was empty.", cname );
		return NULL;
	}

	/* The positions tuple should have the same number of objects as
	 * the motors tuple.
	 */

	num_positions = PyTuple_Size( positions_tuple );

	if ( num_positions != num_motors ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
			"The two tuples passed to '%s' do not contain "
			"the same number of objects.", cname );
		return NULL;
	}

	/* Allocate memory for the motor record array. */

	motor_record_array = (MX_RECORD **) 
				malloc( num_motors * sizeof(MX_RECORD *) );

	if ( motor_record_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
			"%s: Unable to allocate a %d element array "
			"of MX_RECORD pointers.",
			cname, num_motors );
		return NULL;
	}

	/* Allocate memory for the position array. */

	position_array = (double *) malloc( num_motors * sizeof(double ) );

	if ( position_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
			"%s: Unable to allocate a %d element array of doubles.",
			cname, num_motors );

		free( motor_record_array );
		return NULL;
	}

	/* Fill in the motor record array from the motors tuple. */

	for ( i = 0; i < num_motors; i++ ) {
		motor_object = PyTuple_GetItem( motors_tuple, i );

		if ( motor_object == NULL ) {
			FREE_ARRAYS;
			return NULL;
		}

		mx_status = MpPrivate_get_record_from_object(
					motor_object, &motor_record );

		if ( mx_status.code != MXE_SUCCESS ) {
			FREE_ARRAYS;
			return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
		}
		motor_record_array[i] = motor_record;
	}

	/* Copy the position values to the position array. */

	for ( i = 0; i < num_motors; i++ ) {
		position_object = PyTuple_GetItem( positions_tuple, i );

		if ( position_object == NULL ) {
			FREE_ARRAYS;
			return NULL;
		}

		if ( PyFloat_Check( position_object ) == 0 ) {
			MpPrivateErr_SetString( PyExc_TypeError,
	"%s: Element %d of the position tuple is not a floating point number.",
				cname, i );

			FREE_ARRAYS;
			return NULL;
		}
		position_array[i] = PyFloat_AsDouble( position_object );
	}

	/* Finally, we can start the motor moves. */

	mx_status = mx_motor_array_move_absolute( num_motors,
					motor_record_array, position_array, 0 );

	/* Before we do anything else, free the two arrays. */

	FREE_ARRAYS;

	/* If the motor move caused an error to occur, report that. */

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

#undef FREE_ARRAYS

MX_EXPORT PyObject *
MpPrivate_wait_for_multiple_motors_stop_cmd( PyObject *self, PyObject *args )
{
	const char cname[] = "wait_for_multiple_motors_stop";

	PyObject *motors_tuple;
	PyObject *motor_object;
	int i, parse_status, num_motors;
	MX_RECORD **motor_record_array, *motor_record;
	mx_status_type mx_status;

	parse_status = PyArg_ParseTuple( args, "O", &motors_tuple );

	if ( parse_status == 0 )
		return NULL;

	/* The object passed should be a tuple of motor objects. */

	if ( PyTuple_Check( motors_tuple ) == 0 ) {
		MpPrivateErr_SetString( PyExc_TypeError,
		"The argument to '%s' should be a tuple, but it is not.",
			cname );
		return NULL;
	}

	/* Figure out the number of motors by getting the number of objects
	 * in the motor tuple.
	 */

	num_motors = PyTuple_Size( motors_tuple );

	if ( num_motors <= 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
			"The motors tuple passed to '%s' was empty.", cname );
		return NULL;
	}

	/* Allocate memory for the motor record array. */

	motor_record_array = (MX_RECORD **) 
				malloc( num_motors * sizeof(MX_RECORD *) );

	if ( motor_record_array == NULL ) {
		MpPrivateErr_SetString( PyExc_MemoryError,
			"%s: Unable to allocate a %d element array "
			"of MX_RECORD pointers.",
			cname, num_motors );
		return NULL;
	}

	/* Fill in the motor record array from the motors tuple. */

	for ( i = 0; i < num_motors; i++ ) {
		motor_object = PyTuple_GetItem( motors_tuple, i );

		if ( motor_object == NULL ) {
			mx_free( motor_record_array );
			return NULL;
		}

		mx_status = MpPrivate_get_record_from_object(
					motor_object, &motor_record );

		if ( mx_status.code != MXE_SUCCESS ) {
			mx_free( motor_record_array );
			return MpPrivateErr_SetFromMxStatus( NULL, mx_status );
		}
		motor_record_array[i] = motor_record;
	}

	/* Finally, we are ready to wait for the motor moves to end. */

	mx_status = mx_wait_for_motor_array_stop( num_motors,
						motor_record_array, 0 );

	/* Before we do anything else, free the array. */

	mx_free( motor_record_array );

	/* If the motor move caused an error to occur, report that. */

	if ( mx_status.code != MXE_SUCCESS )
		return MpPrivateErr_SetFromMxStatus( NULL, mx_status );

	Py_INCREF(Py_None);

	return Py_None;
}

