/*
 * Name:    mp_output.c
 *
 * Purpose: Python module interface to generic MX output functions.
 *
 * Author:  William Lavender
 *
 *-------------------------------------------------------------------------
 *
 * Copyright 2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include "Python.h"

#include "mx_util.h"
#include "mx_record.h"
#include "mx_handle.h"
#include "mp.h"

/*-------------------------------------------------------------------------*/

static PyObject *MpPrivate_info_output_function = NULL;

static void MpPrivate_info_output_handler( char * );

MX_EXPORT PyObject *
MpPrivate_info_cmd( PyObject *self, PyObject *args )
{
	char *buffer;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "s", &buffer );

	if ( parse_status == 0 )
		return NULL;

	mx_info( buffer );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_set_info_output_function_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_info_output_function";

	PyObject *function_object;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "O", &function_object );

	if ( parse_status == 0 )
		return NULL;

	/* Get rid of any old function object. */

	if ( MpPrivate_info_output_function != NULL ) {
		Py_DECREF( MpPrivate_info_output_function );

		MpPrivate_info_output_function = NULL;
	}

	/* If the argument passed is None, then change the info output
	 * handler back to the default handler and exit.
	 */

	if ( function_object == Py_None ) {
		mx_set_info_output_function( NULL );

		Py_INCREF(Py_None);
		return Py_None;
	}

	/* Is the argument passed a function? */

	if ( PyCallable_Check( function_object ) == 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
"The argument passed to '%s' is not a function or other callable object.",
			cname );

		return NULL;
	}

	/* Save the supplied function argument. */

	MpPrivate_info_output_function = function_object;

	Py_INCREF( MpPrivate_info_output_function );

	/* Install an MX info output handler. */

	mx_set_info_output_function( MpPrivate_info_output_handler );

	Py_INCREF(Py_None);

	return Py_None;
}

static void
MpPrivate_info_output_handler( char *message )
{
	static const char fname[] = "MpPrivate_info_output_handler()";

	PyObject *arglist;
	PyObject *result;

	if ( MpPrivate_info_output_function == NULL ) {
		fprintf( stderr, "%s: Attempted to send the message '%s' "
		"to the MpPrivate_info_output_function pointer, but the "
		"pointer was NULL.", fname, message );

		return;
	}

	arglist = Py_BuildValue("(s)", message );

	if ( arglist == NULL ) {
		fprintf( stderr, "%s: arglist was NULL.\n", fname );

		PyErr_Clear();

		return;
	}

	result = PyEval_CallObject( MpPrivate_info_output_function, arglist );

	Py_DECREF( arglist );

	if ( result == NULL ) {

#if ( PY_MAJOR_VERSION >= 2 )
		PyErr_WriteUnraisable( MpPrivate_info_output_function );
#endif
		PyErr_Clear();

		return;
	}

	Py_DECREF( result );

	return;
}

/*-------------------------------------------------------------------------*/

static PyObject *MpPrivate_warning_output_function = NULL;

static void MpPrivate_warning_output_handler( char * );

MX_EXPORT PyObject *
MpPrivate_warning_cmd( PyObject *self, PyObject *args )
{
	char *buffer;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "s", &buffer );

	if ( parse_status == 0 )
		return NULL;

	mx_warning( buffer );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_set_warning_output_function_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_warning_output_function";

	PyObject *function_object;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "O", &function_object );

	if ( parse_status == 0 )
		return NULL;

	/* Get rid of any old function object. */

	if ( MpPrivate_warning_output_function != NULL ) {
		Py_DECREF( MpPrivate_warning_output_function );

		MpPrivate_warning_output_function = NULL;
	}

	/* If the argument passed is None, then change the warning output
	 * handler back to the default handler and exit.
	 */

	if ( function_object == Py_None ) {
		mx_set_warning_output_function( NULL );

		Py_INCREF(Py_None);
		return Py_None;
	}

	/* Is the argument passed a function? */

	if ( PyCallable_Check( function_object ) == 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
"The argument passed to '%s' is not a function or other callable object.",
			cname );

		return NULL;
	}

	/* Save the supplied function argument. */

	MpPrivate_warning_output_function = function_object;

	Py_INCREF( MpPrivate_warning_output_function );

	/* Install an MX warning output handler. */

	mx_set_warning_output_function( MpPrivate_warning_output_handler );

	Py_INCREF(Py_None);

	return Py_None;
}

static void
MpPrivate_warning_output_handler( char *message )
{
	static const char fname[] = "MpPrivate_warning_output_handler()";

	PyObject *arglist;
	PyObject *result;

	if ( MpPrivate_warning_output_function == NULL ) {
		fprintf( stderr, "%s: Attempted to send the message '%s' "
		"to the MpPrivate_warning_output_function pointer, but the "
		"pointer was NULL.", fname, message );

		return;
	}

	arglist = Py_BuildValue("(s)", message );

	if ( arglist == NULL ) {
		fprintf( stderr, "%s: arglist was NULL.\n", fname );

		PyErr_Clear();

		return;
	}

	result = PyEval_CallObject( MpPrivate_warning_output_function, arglist);

	Py_DECREF( arglist );

	if ( result == NULL ) {

#if ( PY_MAJOR_VERSION >= 2 )
		PyErr_WriteUnraisable( MpPrivate_info_output_function );
#endif
		PyErr_Clear();

		return;
	}

	Py_DECREF( result );

	return;
}

/*-------------------------------------------------------------------------*/

static PyObject *MpPrivate_error_output_function = NULL;

static void MpPrivate_error_output_handler( char * );

#if 0

MX_EXPORT PyObject *
MpPrivate_error_cmd( PyObject *self, PyObject *args )
{
	char *buffer;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "s", &buffer );

	if ( parse_status == 0 )
		return NULL;

	mx_error( buffer );

	Py_INCREF(Py_None);

	return Py_None;
}

#endif

MX_EXPORT PyObject *
MpPrivate_set_error_output_function_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_error_output_function";

	PyObject *function_object;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "O", &function_object );

	if ( parse_status == 0 )
		return NULL;

	/* Get rid of any old function object. */

	if ( MpPrivate_error_output_function != NULL ) {
		Py_DECREF( MpPrivate_error_output_function );

		MpPrivate_error_output_function = NULL;
	}

	/* If the argument passed is None, then change the error output
	 * handler back to the default handler and exit.
	 */

	if ( function_object == Py_None ) {
		mx_set_error_output_function( NULL );

		Py_INCREF(Py_None);
		return Py_None;
	}

	/* Is the argument passed a function? */

	if ( PyCallable_Check( function_object ) == 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
"The argument passed to '%s' is not a function or other callable object.",
			cname );

		return NULL;
	}

	/* Save the supplied function argument. */

	MpPrivate_error_output_function = function_object;

	Py_INCREF( MpPrivate_error_output_function );

	/* Install an MX error output handler. */

	mx_set_error_output_function( MpPrivate_error_output_handler );

	Py_INCREF(Py_None);

	return Py_None;
}

static void
MpPrivate_error_output_handler( char *message )
{
	static const char fname[] = "MpPrivate_error_output_handler()";

	PyObject *arglist;
	PyObject *result;

	if ( MpPrivate_error_output_function == NULL ) {
		fprintf( stderr, "%s: Attempted to send the message '%s' "
		"to the MpPrivate_error_output_function pointer, but the "
		"pointer was NULL.", fname, message );

		return;
	}

	arglist = Py_BuildValue("(s)", message );

	if ( arglist == NULL ) {
		fprintf( stderr, "%s: arglist was NULL.\n", fname );

		PyErr_Clear();

		return;
	}

	result = PyEval_CallObject( MpPrivate_error_output_function, arglist );

	Py_DECREF( arglist );

	if ( result == NULL ) {

#if ( PY_MAJOR_VERSION >= 2 )
		PyErr_WriteUnraisable( MpPrivate_info_output_function );
#endif
		PyErr_Clear();

		return;
	}

	Py_DECREF( result );

	return;
}

/*-------------------------------------------------------------------------*/

static PyObject *MpPrivate_debug_output_function = NULL;

static void MpPrivate_debug_output_handler( char * );

MX_EXPORT PyObject *
MpPrivate_debug_cmd( PyObject *self, PyObject *args )
{
	char *buffer;
	int debug_level, current_debug_level;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "is", &debug_level, &buffer );

	if ( parse_status == 0 )
		return NULL;

	current_debug_level = mx_get_debug_level();

	if ( debug_level <= current_debug_level ) {
		MX_DEBUG( debug_level, ( "%s", buffer ) );
	}

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_set_debug_output_function_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_debug_output_function";

	PyObject *function_object;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "O", &function_object );

	if ( parse_status == 0 )
		return NULL;

	/* Get rid of any old function object. */

	if ( MpPrivate_debug_output_function != NULL ) {
		Py_DECREF( MpPrivate_debug_output_function );

		MpPrivate_debug_output_function = NULL;
	}

	/* If the argument passed is None, then change the debug output
	 * handler back to the default handler and exit.
	 */

	if ( function_object == Py_None ) {
		mx_set_debug_output_function( NULL );

		Py_INCREF(Py_None);
		return Py_None;
	}

	/* Is the argument passed a function? */

	if ( PyCallable_Check( function_object ) == 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
"The argument passed to '%s' is not a function or other callable object.",
			cname );

		return NULL;
	}

	/* Save the supplied function argument. */

	MpPrivate_debug_output_function = function_object;

	Py_INCREF( MpPrivate_debug_output_function );

	/* Install an MX debug output handler. */

	mx_set_debug_output_function( MpPrivate_debug_output_handler );

	Py_INCREF(Py_None);

	return Py_None;
}

static void
MpPrivate_debug_output_handler( char *message )
{
	static const char fname[] = "MpPrivate_debug_output_handler()";

	PyObject *arglist;
	PyObject *result;

	if ( MpPrivate_debug_output_function == NULL ) {
		fprintf( stderr, "%s: Attempted to send the message '%s' "
		"to the MpPrivate_debug_output_function pointer, but the "
		"pointer was NULL.", fname, message );

		return;
	}

	arglist = Py_BuildValue("(s)", message );

	if ( arglist == NULL ) {
		fprintf( stderr, "%s: arglist was NULL.\n", fname );

		PyErr_Clear();

		return;
	}

	result = PyEval_CallObject( MpPrivate_debug_output_function, arglist );

	Py_DECREF( arglist );

	if ( result == NULL ) {

#if ( PY_MAJOR_VERSION >= 2 )
		PyErr_WriteUnraisable( MpPrivate_info_output_function );
#endif
		PyErr_Clear();

		return;
	}

	Py_DECREF( result );

	return;
}

/*-------------------------------------------------------------------------*/

static PyObject *MpPrivate_info_dialog_function = NULL;

static void MpPrivate_info_dialog_handler( char *, char *, char * );

MX_EXPORT PyObject *
MpPrivate_info_dialog_cmd( PyObject *self, PyObject *args )
{
	char *text_prompt, *gui_prompt, *button_label;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "sss",
				&text_prompt, &gui_prompt, &button_label );

	if ( parse_status == 0 )
		return NULL;

	mx_info_dialog( text_prompt, gui_prompt, button_label );

	Py_INCREF(Py_None);

	return Py_None;
}

MX_EXPORT PyObject *
MpPrivate_set_info_dialog_function_cmd( PyObject *self, PyObject *args )
{
	static const char cname[] = "set_info_dialog_function";

	PyObject *function_object;
	int parse_status;

	parse_status = PyArg_ParseTuple( args, "O", &function_object );

	if ( parse_status == 0 )
		return NULL;

	/* Get rid of any old function object. */

	if ( MpPrivate_info_dialog_function != NULL ) {
		Py_DECREF( MpPrivate_info_dialog_function );

		MpPrivate_info_dialog_function = NULL;
	}

	/* If the argument passed is None, then change the info dialog
	 * handler back to the default handler and exit.
	 */

	if ( function_object == Py_None ) {
		mx_set_info_dialog_function( NULL );

		Py_INCREF(Py_None);
		return Py_None;
	}

	/* Is the argument passed a function? */

	if ( PyCallable_Check( function_object ) == 0 ) {
		MpPrivateErr_SetString( PyExc_RuntimeError,
"The argument passed to '%s' is not a function or other callable object.",
			cname );

		return NULL;
	}

	/* Save the supplied function argument. */

	MpPrivate_info_dialog_function = function_object;

	Py_INCREF( MpPrivate_info_dialog_function );

	/* Install an MX info_dialog output handler. */

	mx_set_info_dialog_function( MpPrivate_info_dialog_handler );

	Py_INCREF(Py_None);

	return Py_None;
}

static void
MpPrivate_info_dialog_handler( char *text_prompt,
				char *gui_prompt,
				char *button_label )
{
	static const char fname[] = "MpPrivate_info_dialog_handler()";

	PyObject *arglist;
	PyObject *result;

	if ( MpPrivate_info_dialog_function == NULL ) {
		fprintf( stderr, "%s: Attempted to invoke the "
		"MpPrivate_info_dialog_function pointer, but the "
		"pointer was NULL."
		"text_prompt = '%s', gui_prompt = '%s', button_label = '%s'",
		fname, text_prompt, gui_prompt, button_label );

		return;
	}

	arglist = Py_BuildValue("(sss)", text_prompt, gui_prompt, button_label);

	if ( arglist == NULL ) {
		fprintf( stderr, "%s: arglist was NULL.\n", fname );

		PyErr_Clear();

		return;
	}

	result = PyEval_CallObject( MpPrivate_info_dialog_function, arglist );

	Py_DECREF( arglist );

	if ( result == NULL ) {

#if ( PY_MAJOR_VERSION >= 2 )
		PyErr_WriteUnraisable( MpPrivate_info_output_function );
#endif
		PyErr_Clear();

		return;
	}

	Py_DECREF( result );

	return;
}

